==> mule-module-api-deployment-4.3.0-20220919-sources.jar: com/mulesoft/mule/runtime/gw/deployment/tracking/ApiTracker.java <==
/*
 * (c) 2003-2020 MuleSoft, Inc. This software is protected under international copyright law. All use of this software is subject to
 * MuleSoft's Master Subscription Agreement (or other Terms of Service) separately entered into between you and MuleSoft. If such an
 * agreement is not in place, you may not use the software.
 */
package com.mulesoft.mule.runtime.gw.deployment.tracking;

import static com.mulesoft.mule.runtime.gw.api.logging.ExceptionDescriptor.errorMessage;

import org.mule.runtime.api.lifecycle.Disposable;

import com.mulesoft.mule.runtime.gw.api.key.ApiKey;
import com.mulesoft.mule.runtime.gw.client.exception.IncompatibleApiException;
import com.mulesoft.mule.runtime.gw.client.exception.NormalizedExceptionMessageLogger;
import com.mulesoft.mule.runtime.gw.client.exception.NotFoundException;
import com.mulesoft.mule.runtime.gw.client.exception.RecoverableExceptionMessageLogger;
import com.mulesoft.mule.runtime.gw.client.model.ApiResponse;
import com.mulesoft.mule.runtime.gw.client.session.factory.ApiPlatformSessionFactory;
import com.mulesoft.mule.runtime.gw.logging.GatewayMuleAppLoggerFactory;
import com.mulesoft.mule.runtime.gw.model.Api;
import com.mulesoft.mule.runtime.gw.model.ApiImplementation;
import com.mulesoft.mule.runtime.gw.notification.ApiDeploymentListener;
import com.mulesoft.mule.runtime.gw.retry.RunnableRetrier;

import org.slf4j.Logger;

public class ApiTracker implements ApiDeploymentListener, Disposable {

  private static final String TRACK_API_DESCRIPTION = "retrieve API information for API";
  private static final Logger LOGGER = GatewayMuleAppLoggerFactory.getLogger(ApiTracker.class);

  private final NormalizedExceptionMessageLogger normalizedLogger = new RecoverableExceptionMessageLogger(LOGGER);

  private final ApiTrackingService apiTrackingService;
  private final ApiPlatformSessionFactory platformSessionFactory;
  private final RunnableRetrier<ApiKey> runnableRetrier;

  public ApiTracker(ApiTrackingService apiTrackingService,
                    ApiPlatformSessionFactory platformSessionFactory,
                    RunnableRetrier<ApiKey> runnableRetrier) {
    this.apiTrackingService = apiTrackingService;
    this.platformSessionFactory = platformSessionFactory;
    this.runnableRetrier = runnableRetrier;
  }

  @Override
  public void onApiDeploymentSuccess(Api api) {
    try {
      trackApi(api);
    } catch (Throwable t) {
      scheduleRetry(api);
    }
  }

  public void trackApi(Api api) {
    if (api.isOffline()) {
      LOGGER.debug("Tracking skipped for offline API {}", api.getKey());
      return;
    }
    ApiImplementation implementation = api.getImplementation();
    try {
      LOGGER.debug("Tracking API {}", implementation.getApiKey());

      ApiResponse apiResponse = platformSessionFactory.create().getApi(implementation.getApiKey());

      apiTrackingService.apiTracked(implementation.getApiKey(), apiResponse.getTrackingInfo(), apiResponse.getPolicySet(),
                                    apiResponse.getSlas());

      LOGGER.debug("API {} successfully tracked. API name is {} and version is {} ", implementation.getApiKey(),
                   apiResponse.getTrackingInfo().getInstanceName(), apiResponse.getTrackingInfo().getVersion());

    } catch (NotFoundException e) {
      LOGGER.warn("API with ID {} does not exist in API Platform for the current organization.", implementation.getApiKey().id());
      apiTrackingService.apiUntracked(implementation.getApiKey());
    } catch (IncompatibleApiException e) {
      LOGGER.error("There was an error trying to track API {}. {}", implementation.getApiKey().id(), errorMessage(e));
    } catch (Throwable t) {
      normalizedLogger.error(TRACK_API_DESCRIPTION, implementation.getApiKey().id(), t);
      apiTrackingService.apiTrackingFailed(implementation.getApiKey());
      throw t;
    }
  }

  private void scheduleRetry(Api api) {
    runnableRetrier.scheduleRetry(api.getKey(), () -> trackApi(api));
  }

  @Override
  public void dispose() {
    runnableRetrier.dispose();
  }
}

==> mule-module-api-deployment-4.5.0-20220622-sources.jar: com/mulesoft/mule/runtime/gw/deployment/tracking/ApiTracker.java <==
/*
 * (c) 2003-2020 MuleSoft, Inc. This software is protected under international copyright law. All use of this software is subject to
 * MuleSoft's Master Subscription Agreement (or other Terms of Service) separately entered into between you and MuleSoft. If such an
 * agreement is not in place, you may not use the software.
 */
package com.mulesoft.mule.runtime.gw.deployment.tracking;

import static com.mulesoft.mule.runtime.gw.api.logging.ExceptionDescriptor.errorMessage;

import com.mulesoft.mule.runtime.gw.client.exception.IncompatibleApiException;
import com.mulesoft.mule.runtime.gw.client.exception.NonRecoverableExceptionMessageLogger;
import com.mulesoft.mule.runtime.gw.client.exception.NormalizedExceptionMessageLogger;
import com.mulesoft.mule.runtime.gw.client.exception.NotFoundException;
import com.mulesoft.mule.runtime.gw.client.model.ApiResponse;
import com.mulesoft.mule.runtime.gw.client.session.factory.ApiPlatformSessionFactory;
import com.mulesoft.mule.runtime.gw.logging.GatewayMuleAppLoggerFactory;
import com.mulesoft.mule.runtime.gw.model.Api;
import com.mulesoft.mule.runtime.gw.model.ApiImplementation;
import com.mulesoft.mule.runtime.gw.notification.ApiDeploymentListener;

import java.util.function.Function;

import org.slf4j.Logger;

public class ApiTracker implements ApiDeploymentListener {

  private static final String TRACK_API_DESCRIPTION = "retrieve API information for API";
  private static final Logger LOGGER = GatewayMuleAppLoggerFactory.getLogger(ApiTracker.class);

  private final NormalizedExceptionMessageLogger normalizedLogger;

  private final ApiTrackingService apiTrackingService;

  private final ApiPlatformSessionFactory platformSessionFactory;

  public ApiTracker(ApiTrackingService apiTrackingService,
                    ApiPlatformSessionFactory platformSessionFactory) {
    this(apiTrackingService, platformSessionFactory, NonRecoverableExceptionMessageLogger::new);
  }

  public ApiTracker(ApiTrackingService apiTrackingService,
                    ApiPlatformSessionFactory platformSessionFactory,
                    Function<Logger, NormalizedExceptionMessageLogger> loggerSupplier) {
    this.apiTrackingService = apiTrackingService;
    this.platformSessionFactory = platformSessionFactory;
    this.normalizedLogger = loggerSupplier.apply(LOGGER);
  }

  @Override
  public void onApiDeploymentSuccess(Api api) {
    trackApi(api);
  }

  protected void trackApi(Api api) {
    if (api.isOffline()) {
      LOGGER.debug("Tracking skipped for offline API {}", api.getKey());
      return;
    }
    ApiImplementation implementation = api.getImplementation();
    try {
      LOGGER.debug("Tracking API {}", implementation.getApiKey());

      ApiResponse apiResponse = platformSessionFactory.create().getApi(implementation.getApiKey());

      apiTrackingService.apiTracked(implementation.getApiKey(), apiResponse.getTrackingInfo(), apiResponse.getPolicySet(),
                                    apiResponse.getSlas());

      LOGGER.debug("API {} successfully tracked. API name is {} and version is {} ", implementation.getApiKey(),
                   apiResponse.getTrackingInfo().getInstanceName(), apiResponse.getTrackingInfo().getVersion());

    } catch (NotFoundException e) {
      LOGGER.warn("API with ID {} does not exist in API Platform for the current organization.", implementation.getApiKey().id());
      apiTrackingService.apiUntracked(implementation.getApiKey());
    } catch (IncompatibleApiException e) {
      LOGGER.error("There was an error trying to track API {}. {}", implementation.getApiKey().id(), errorMessage(e));
    } catch (Throwable t) {
      normalizedLogger.error(TRACK_API_DESCRIPTION, implementation.getApiKey().id(), t);
      apiTrackingService.apiTrackingFailed(implementation.getApiKey());
      throw t;
    }
  }
}
